@@ -14,7 +14,7 @@ from urllib import urlencode
 
 from openid.consumer.consumer import Consumer, SUCCESS, CANCEL, FAILURE
 from openid.consumer.discover import DiscoveryFailure
-from openid.extensions import sreg, ax
+from openid.extensions import sreg, ax, pape
 
 from oauth2 import Consumer as OAuthConsumer, Token, Request as OAuthRequest
 
@@ -357,9 +357,11 @@ class BaseAuth(object):
         args = args[:] + tuple(map(ctype_to_model, session_data['args']))
 
         kwargs = kwargs.copy()
-        kwargs.update((key, ctype_to_model(val))
+        saved_kwargs = dict((key, ctype_to_model(val))
                             for key, val in session_data['kwargs'].iteritems())
-        return (session_data['next'], args, kwargs)
+        saved_kwargs.update((key, val)
+                            for key, val in kwargs.iteritems())
+        return (session_data['next'], args, saved_kwargs)
 
     def continue_pipeline(self, *args, **kwargs):
         """Continue previous halted pipeline"""
@@ -381,9 +383,14 @@ class BaseAuth(object):
         """Return extra arguments needed on auth process, setting is per
         backend and defined by:
             <backend name in uppercase>_AUTH_EXTRA_ARGUMENTS.
+        The defaults can be overriden by GET parameters.
         """
         backend_name = self.AUTH_BACKEND.name.upper().replace('-', '_')
-        return setting(backend_name + '_AUTH_EXTRA_ARGUMENTS', {})
+        extra_arguments = setting(backend_name + '_AUTH_EXTRA_ARGUMENTS', {})
+        for key in extra_arguments:
+            if key in self.data:
+                extra_arguments[key] = self.data[key]
+        return extra_arguments
 
     @property
     def uses_redirect(self):
@@ -486,6 +493,16 @@ class OpenIdAuth(BaseAuth):
             fetch_request = sreg.SRegRequest(optional=dict(SREG_ATTR).keys())
         openid_request.addExtension(fetch_request)
 
+        # Add PAPE Extension for max_auth_age, if configured
+        max_age = setting('SOCIAL_AUTH_OPENID_PAPE_MAX_AUTH_AGE')
+        if max_age is not None:
+            try:
+                openid_request.addExtension(
+                    pape.Request(max_auth_age=int(max_age))
+                )
+            except (ValueError, TypeError):
+                pass
+
         return openid_request
 
     def consumer(self):
